@@ -22,6 +22,7 @@ from typing import (
     Literal,
     NoReturn,
     Protocol,
+    Tuple,
     TypeVar,
     cast,
 )
@@ -41,6 +42,7 @@ from stravalib.exc import (
     warn_param_unofficial,
     warn_param_unsupported,
 )
+from stravalib.model import SummaryAthlete
 from stravalib.protocol import AccessInfo, ApiV3, Scope
 from stravalib.util import limiter
 
@@ -67,6 +69,8 @@ class Client:
         rate_limit_requests: bool = True,
         rate_limiter: limiter.RateLimiter | None = None,
         requests_session: Session | None = None,
+        token_expires: int | None = None,
+        refresh_token: str | None = None,
     ) -> None:
         """
         Initialize a new client object.
@@ -84,6 +88,12 @@ class Client:
             :class:`stravalib.util.limiter.DefaultRateLimiter` will be used.
         requests_session : requests.Session() object
             (Optional) pass request session object.
+        token_expires : int
+            epoch timestamp -- seconds since jan 1 1970 (Epoch timestamp)
+            This represents the date and time that the token will expire. It is
+            used to automatically check and refresh the token in the client
+            method on all API requests.
+        refresh_token : str
 
         """
         self.log = logging.getLogger(
@@ -101,6 +111,8 @@ class Client:
 
         self.protocol = ApiV3(
             access_token=access_token,
+            refresh_token=refresh_token,
+            token_expires=token_expires,
             requests_session=requests_session,
             rate_limiter=rate_limiter,
         )
@@ -111,14 +123,13 @@ class Client:
         return self.protocol.access_token
 
     @access_token.setter
-    def access_token(self, token_value: str) -> None:
+    def access_token(self, token_value: str | None) -> None:
         """Set the currently configured authorization token.
 
         Parameters
         ----------
-        token_value : int
-             User's access token for authentication.
-
+        access_token : str
+             User's access token
 
         Returns
         -------
@@ -126,6 +137,48 @@ class Client:
         """
         self.protocol.access_token = token_value
 
+    @property
+    def token_expires(self) -> int | None:
+        """The currently configured authorization token."""
+        return self.protocol.token_expires
+
+    @token_expires.setter
+    def token_expires(self, expires_value: int) -> None:
+        """Used to set and update the refresh token.
+
+        Parameters
+        ----------
+        expires_value : int
+             Current token expiration time (epoch seconds)
+
+        Returns
+        -------
+
+        """
+        self.protocol.token_expires = expires_value
+
+    @property
+    def refresh_token(self) -> str | None:
+        """The currently configured authorization token."""
+        return self.protocol.refresh_token
+
+    @refresh_token.setter
+    def refresh_token(self, refresh_value: str) -> None:
+        """Used to set and update the refresh token.
+
+        Parameters
+        ----------
+        refresh_value : str
+             Current token refresh value.
+
+        Returns
+        -------
+        None
+            Updates the `refresh_token` attribute in the Client class.
+
+        """
+        self.protocol.refresh_token = refresh_value
+
     def authorization_url(
         self,
         client_id: int,
@@ -175,8 +228,12 @@ class Client:
         )
 
     def exchange_code_for_token(
-        self, client_id: int, client_secret: str, code: str
-    ) -> AccessInfo:
+        self,
+        client_id: int,
+        client_secret: str,
+        code: str,
+        return_athlete: bool = False,
+    ) -> AccessInfo | Tuple[AccessInfo, SummaryAthlete | None]:
         """Exchange the temporary authorization code (returned with redirect
         from Strava authorization URL) for a short-lived access token and a
         refresh token (used to obtain the next access token later on).
@@ -189,18 +246,46 @@ class Client:
             The developer client secret
         code : str
             The temporary authorization code
+        return_athlete : bool (default = False)
+            Whether to return a SummaryAthlete object (or not)
+            This parameter is currently undocumented and could change
+            at any time.
 
         Returns
         -------
-        dict
-            Dictionary containing the access_token, refresh_token and
+        AccessInfo
+            TypedDictionary containing the access_token, refresh_token and
             expires_at (number of seconds since Epoch when the provided access
             token will expire)
+        tuple
+            Contains:
+             - the `AccessInfo` typed dict containing token values
+             - a `SummaryAthlete` object representing the authenticated user.
+
+        Notes
+        -----
+        Strava by default returns `SummaryAthlete` information during
+        this exchange. However this return is currently undocumented
+        and could change at any time.
         """
-        return self.protocol.exchange_code_for_token(
-            client_id=client_id, client_secret=client_secret, code=code
+        access_info, athlete_data = self.protocol.exchange_code_for_token(
+            client_id=client_id,
+            client_secret=client_secret,
+            code=code,
+            return_athlete=return_athlete,
         )
 
+        # Return both access_info and athlete if requested. Athlete will be None if Strava
+        # doesn't return it in their end point response.
+        if return_athlete:
+            if athlete_data:
+                summary_athlete = SummaryAthlete.model_validate(athlete_data)
+            else:
+                summary_athlete = None
+            return access_info, summary_athlete
+        else:
+            return access_info
+
     def refresh_access_token(
         self, client_id: int, client_secret: str, refresh_token: str
     ) -> AccessInfo:
@@ -1250,8 +1335,6 @@ class Client:
     def get_segment_effort(self, effort_id: int) -> model.SegmentEffort:
         """Return a specific segment effort by ID.
 
-        https://developers.strava.com/docs/reference/#api-SegmentEfforts
-
         Parameters
         ----------
         effort_id : int
@@ -1262,6 +1345,10 @@ class Client:
         class:`stravalib.model.SegmentEffort`
             The specified effort on a segment.
 
+        Notes
+        ------
+        https://developers.strava.com/docs/reference/#api-SegmentEfforts
+
         """
         return model.SegmentEffort.model_validate(
             self.protocol.get("/segment_efforts/{id}", id=effort_id)
@@ -1358,6 +1445,7 @@ class Client:
             limit=limit,
         )
 
+    # Next TODO: add tests to deprecated items
     def get_segment_efforts(
         self,
         segment_id: int,
@@ -1369,49 +1457,62 @@ class Client:
         """Gets all efforts on a particular segment sorted by start_date_local
 
         Returns an array of segment effort summary representations sorted by
-        start_date_local ascending or by elapsed_time if an athlete_id is
+        `start_date_local` ascending or by elapsed_time if an athlete_id is
         provided.
 
-        If no filtering parameters is provided all efforts for the segment
+        NOTES:
+        * The `athlete_id` sorting only works if you do NOT provide start and end dates.
+        * If no date filtering parameters are provided, all efforts for the segment
         will be returned.
 
         Date range filtering is accomplished using an inclusive start and end
-        time, thus start_date_local and end_date_local must be sent together.
+        time. `start_date_local` and `end_date_local` must be sent together.
         For open ended ranges pick dates significantly in the past or future.
         The filtering is done over local time for the segment, so there is no
-        need for timezone conversion. For example, all efforts on Jan. 1st, 2014
+        need for timezone conversion. For example, all efforts on Jan. 1st, 2024
         for a segment in San Francisco, CA can be fetched using
-        2014-01-01T00:00:00Z and 2014-01-01T23:59:59Z.
+        `2024-01-01T00:00:00Z` and `2024-01-01T23:59:59Z`.
 
-        https://developers.strava.com/docs/reference/#api-SegmentEfforts-getEffortsBySegmentId
+        This endpoint requires a Strava subscription.
 
         Parameters
         ----------
         segment_id : int
             ID for the segment of interest
-        athlete_id: int, optional
-            ID of athlete.
-        start_date_local : datetime.datetime or str, optional, default=None
+        start_date_local : `datetime.datetime` or `str`, optional, default=None
             Efforts before this date will be excluded.
             Either as ISO8601 or datetime object
-        end_date_local : datetime.datetime or str, optional, default=None
+        end_date_local : `datetime.datetime` or str, optional, default=None
             Efforts after this date will be excluded.
             Either as ISO8601 or datetime object
         limit : int, default=None, optional
             limit number of efforts.
+            This parameter has been removed by Strava and will be removed in
+            stravalib in the near future.
+            .. deprecated::
+                This param is not supported by the Strava API and may be
+                removed in the future.
         athlete_id : int, default=None
-            Strava ID for the athlete
-
+            Strava ID for the athlete. Used to sort efforts by duration.
+            Can only be used if start and end date are not provided.
+            .. deprecated::
+                This param is not supported by the Strava API and may be
+                removed in the future.
         Returns
         -------
         class:`BatchedResultsIterator`
             An iterator of :class:`stravalib.model.SegmentEffort` efforts on
             a segment.
 
+        Notes
+        ------
+        https://developers.strava.com/docs/reference/#api-SegmentEfforts-getEffortsBySegmentId
+
         """
         params: dict[str, Any] = {"segment_id": segment_id}
 
         if athlete_id is not None:
+            warn_param_unsupported("athlete_id")
             params["athlete_id"] = athlete_id
 
         if start_date_local:
@@ -1429,10 +1530,13 @@ class Client:
             )
 
         if limit is not None:
+            warn_param_deprecation("limit", "date ranges")
             params["limit"] = limit
 
         result_fetcher = functools.partial(
-            self.protocol.get, "/segments/{segment_id}/all_efforts", **params
+            self.protocol.get,
+            "/segment_efforts",
+            **params,
         )
 
         return BatchedResultsIterator(
