@@ -1,19 +1,26 @@
 import datetime
 import json
 import os
+import warnings
 from unittest import mock
+from unittest.mock import patch
 
 import pytest
 import responses
 from responses import matchers
 
 from stravalib.client import ActivityUploader
-from stravalib.exc import AccessUnauthorized, ActivityPhotoUploadFailed
+from stravalib.exc import (
+    AccessUnauthorized,
+    ActivityPhotoUploadFailed,
+)
 from stravalib.model import DetailedAthlete, SummaryAthlete, SummarySegment
 from stravalib.strava_model import SummaryActivity, Zones
 from stravalib.tests import RESOURCES_DIR
 from stravalib.unit_helper import miles
 
+warnings.simplefilter("always")
+
 
 @pytest.fixture
 def zone_response():
@@ -49,6 +56,32 @@ def default_request_params():
     return default_request_params
 
 
+@pytest.fixture
+def raw_exchange_response():
+    """Expected response from the protocol exchange_code_for_token method."""
+    return (
+        {
+            "access_token": "123456",
+            "refresh_token": "789sdf987",
+            "expires_at": 1732417459,
+        },
+        {
+            "id": 10295934,
+            "username": "foo_bar",
+            "resource_state": 2,
+            "firstname": "Foo",
+            "lastname": "Bar",
+            "bio": "A bio",
+            "city": "City",
+            "state": "State",
+            "country": "Country",
+            "sex": "F",
+            "premium": True,
+            "summit": True,
+        },
+    )
+
+
 def test_get_athlete(mock_strava_api, client):
     mock_strava_api.get("/athlete", response_update={"id": 42})
     athlete = client.get_athlete()
@@ -885,6 +918,50 @@ def test_get_segment_effort(mock_strava_api, client):
     assert effort.max_heartrate == 170
 
 
+@pytest.mark.parametrize(
+    "params, warning_type, warning_message",
+    [
+        (
+            {"segment_id": 2345, "athlete_id": 12345},
+            DeprecationWarning,
+            'The "athlete_id" parameter is unsupported',
+        ),
+        (
+            {"segment_id": 2345, "limit": 10},
+            DeprecationWarning,
+            'The "limit" parameter is deprecated',
+        ),
+    ],
+)
+def test_get_segment_efforts_warnings(
+    params, warning_type, warning_message, mock_strava_api, client
+):
+    """Test that if provided with deprecated params, the user receives a
+    warning."""
+
+    mock_strava_api.get(
+        "/segment_efforts",
+        response_update={"name": "Best Run Ever"},
+        n_results=1,
+    )
+
+    with pytest.warns(warning_type, match=warning_message):
+        a = client.get_segment_efforts(**params)
+        next(a)
+
+
+def test_get_segment_efforts(client, mock_strava_api):
+    """Test that endpoint returns data as expected."""
+    mock_strava_api.get("/segment_efforts", n_results=4)
+
+    efforts = list(
+        client.get_segment_efforts(segment_id=2345, athlete_id=12345)
+    )
+
+    assert len(efforts) == 4
+    assert efforts[0].name == "Alpe d'Huez"
+
+
 def test_get_activities_paged(mock_strava_api, client):
     for i in range(1, 4):
         params = {"page": i, "per_page": 200}
@@ -1034,3 +1111,60 @@ def test_get_activity_kudos(mock_strava_api, client):
     assert isinstance(kudoer_list[0], SummaryAthlete)
     assert len(kudoer_list) == 2
     assert kudoer_list[0].lastname == "Doe"
+
+
+@patch("stravalib.protocol.ApiV3.exchange_code_for_token")
+def test_exchange_code_for_token_athlete(
+    mock_protocol_exchange_code, client, raw_exchange_response
+):
+    """If a user requests athlete data, then it should be returned as a
+    SummaryAthlete object."""
+    mock_protocol_exchange_code.return_value = raw_exchange_response
+
+    access_info, summary_athlete = client.exchange_code_for_token(
+        client_id=123,
+        client_secret="secret",
+        code="temp_code",
+        return_athlete=True,
+    )
+
+    assert access_info["access_token"] == "123456"
+    assert summary_athlete.firstname == "Foo"
+
+
+@patch("stravalib.protocol.ApiV3.exchange_code_for_token")
+def test_exchange_code_for_token_missing_athlete(
+    mock_protocol_exchange_code, client, raw_exchange_response
+):
+    """If athlete=True but Strava modifies the api response and athlete return
+    is None client should only return AccessInfo object"""
+    mock_protocol_exchange_code.return_value = (raw_exchange_response[0], None)
+    access_info, athlete = client.exchange_code_for_token(
+        client_id=123,
+        client_secret="secret",
+        code="temp_code",
+        return_athlete=True,
+    )
+
+    assert athlete is None
+    assert access_info["access_token"] == "123456"
+
+
+@patch("stravalib.protocol.ApiV3.exchange_code_for_token")
+def test_exchange_code_for_token_no_athlete(
+    mock_protocol_exchange_code, client, raw_exchange_response
+):
+    """When athlete isn't True, only return AccessInfo Typed Dict."""
+    mock_protocol_exchange_code.return_value = raw_exchange_response
+
+    access_info = client.exchange_code_for_token(
+        client_id=123,
+        client_secret="secret",
+        code="temp_code",
+        return_athlete=False,
+    )
+
+    assert (
+        access_info["access_token"] == raw_exchange_response[0]["access_token"]
+    )
+    assert isinstance(access_info, dict)
